/*
 * arch/arm/plat-spear/include/plat/memory.h
 *
 * Memory map for SPEAr platform
 *
 * Copyright (C) 2009 ST Microelectronics
 * Viresh Kumar<viresh.kumar@st.com>
 *
 * This file is licensed under the terms of the GNU General Public
 * License version 2. This program is licensed "as is" without any
 * warranty of any kind, whether express or implied.
 */

#ifndef __PLAT_MEMORY_H
#define __PLAT_MEMORY_H

/* Physical DRAM offset */
#define PLAT_PHYS_OFFSET		UL(0x00000000)

#endif /* __PLAT_MEMORY_H */
